
from dataclasses import dataclass


@dataclass(frozen=True)
class Offset:
    """An offset in a two dimensional numpy array"""
    dy: int = 0
    dx: int = 0

    def __add__(self, other):
        if isinstance(other, Offset):
            return Offset(self.dy + other.dy, self.dx + other.dx)
        else:
            return Offset(self.dy + int(other), self.dx + int(other))

    def __sub__(self, other):
        if isinstance(other, Offset):
            return Offset(self.dy - other.dy, self.dx - other.dx)
        else:
            return Offset(self.dy - int(other), self.dx - int(other))

    def __mul__(self, other):
        if isinstance(other, Offset):
            return Offset(self.dy * other.dy, self.dx * other.dx)
        else:
            return Offset(self.dy * int(other), self.dx * int(other))

    def is_linear(self) -> bool:
        return self.dy == 0 or self.dx == 0

    def is_diagonal(self) -> bool:
        return self.dy != 0 and self.dx != 0


UP = Offset(dy=-1, dx=0)
DOWN = Offset(dy=1, dx=0)
LEFT = Offset(dy=0, dx=-1)
RIGHT = Offset(dy=0, dx=1)

UP_LEFT = UP + LEFT
UP_RIGHT = UP + RIGHT
DOWN_LEFT = DOWN + LEFT
DOWN_RIGHT = DOWN + RIGHT

VERTICAL = {UP, DOWN}
HORIZONTAL = {LEFT, RIGHT}
LINEAR = VERTICAL | HORIZONTAL
DIAGONAL = {UP_LEFT, UP_RIGHT, DOWN_LEFT, DOWN_RIGHT}
ALL = LINEAR | DIAGONAL








from abc import ABC, abstractmethod

from typing import Set

from chess.color import Color
from chess.piece import ChessPiece
from chess.path import Path


class Unit(ChessPiece, ABC):
    color: Color
    paths: Set[Path]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.has_moved = False





from typing import Set

from chess.color import Color
from chess.path import Path, INFINITE_STEPS
from chess.offset import LINEAR
from chess.piece import Rook
from chess.unit import Unit


ROOK_PATHS: Set[Path] = {Path(offset=offset, max_steps=INFINITE_STEPS) for offset in LINEAR}


class WhiteRook(Unit, Rook):
    color: Color = Color.WHITE
    paths: Set[Path] = ROOK_PATHS


class BlackRook(Unit, Rook):
    color: Color = Color.BLACK
    paths: Set[Path] = ROOK_PATHS




from chess.color import Color
from chess.move import Move
from chess.notation import AlgebraicNotation


class FIDE(AlgebraicNotation):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)






from abc import ABC, abstractmethod
from chess.move import Move


class Notation:
    def __init__(self, move: Move, *args, **kwargs):
        self.move: Move = move

    def __str__(self):
        return self._get_notation_string()

    @classmethod
    def get_move_from_string(cls, notation: str):
        raise NotImplementedError()

    def _get_notation_string(self):
        raise NotImplementedError()








from typing import Set

from chess.color import Color
from chess.path import Path, INFINITE_STEPS
from chess.offset import LINEAR, DIAGONAL
from chess.piece import Queen
from chess.unit import Unit


QUEEN_PATHS: Set[Path] = {Path(offset=offset, max_steps=INFINITE_STEPS) for offset in LINEAR | DIAGONAL}


class WhiteQueen(Unit, Queen):
    color: Color = Color.WHITE
    paths: Set[Path] = QUEEN_PATHS


class BlackQueen(Unit, Queen):
    color: Color = Color.BLACK
    paths: Set[Path] = QUEEN_PATHS




from typing import List

import numpy as np

from chess.color import Color
from chess.unit import (
    Unit,
    BlackKing,
    BlackQueen,
    BlackRook,
    BlackBishop,
    BlackKnight,
    BlackPawn,
    WhiteKing,
    WhiteQueen,
    WhiteRook,
    WhiteBishop,
    WhiteKnight,
    WhitePawn,
)
from chess.square import Square


standard_black_pieces = [
    BlackRook, BlackKnight, BlackBishop, BlackQueen, BlackKing, BlackBishop, BlackKnight, BlackRook,
    BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn,

]

standard_white_pieces = [
    WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn,
    WhiteRook, WhiteKnight, WhiteBishop, WhiteQueen, WhiteKing, WhiteBishop, WhiteKnight, WhiteRook,
]

standard_pieces = [*standard_black_pieces, *standard_white_pieces]


standard_board_array = np.array(
    [[BlackRook, BlackKnight, BlackBishop, BlackQueen, BlackKing, BlackBishop, BlackKnight, BlackRook],
     [BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn, BlackPawn],
     [None, None, None, None, None, None, None, None],
     [None, None, None, None, None, None, None, None],
     [None, None, None, None, None, None, None, None],
     [None, None, None, None, None, None, None, None],
     [WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn, WhitePawn],
     [WhiteRook, WhiteKnight, WhiteBishop, WhiteQueen, WhiteKing, WhiteBishop, WhiteKnight, WhiteRook]]
)
class ChessBoard:
    def __init__(self, array: np.ndarray = standard_board_array, *args, **kwargs):
        self._array: np.ndarray = array

    def __eq__(self, other):
        if isinstance(other, ChessBoard):
            return np.array_equal(self._array, other._array)
        elif isinstance(other, np.ndarray):
            return np.array_equal(self._array, other)
        else:
            return False

    def get(self, square: Square) -> Unit | None:
        return self._array[square.row][square.column]

    def set(self, square: Square, unit: Unit | None):
        self._array[square.row][square.column] = unit

    def get_height(self) -> int:
        return self._array.shape[0]

    def get_width(self) -> int:
        return self._array.shape[1]

    def get_max_y_index(self) -> int:
        return self.get_height() - 1

    def get_max_x_index(self) -> int:
        return self.get_width() - 1

    def get_units(self) -> List[Unit]:
        units = []
        for square in self._array.flatten():
            if square is not None:
                units.append(square)
        return units

    def get_absolute_max_path_steps(self) -> int:
        return max(self.get_height(), self.get_width())

    def is_valid_square(self, square: Square) -> bool:
        return 0 <= square.row < self.get_height() and 0 <= square.column < self.get_width()






from dataclasses import dataclass

from chess.offset import Offset


@dataclass(frozen=True)
class Square:
    row: int
    column: int

    def offset(self, offset: Offset) -> 'Square':
        return Square(self.row + offset.dy, self.column + offset.dx)





from dataclasses import dataclass
import chess.piece

from chess.square import Square


@dataclass
class Move:
    piece: chess.piece.ChessPiece
    from_square: Square
    to_square: Square
    captured_piece: chess.piece.ChessPiece = None
    promotion: chess.piece.ChessPiece = None
    check: bool = False






import chess.board
import chess.unit

from chess.move import Move
from chess.path import Path, INFINITE_STEPS, AllowedMovementTypes
from chess.square import Square
from chess.unit import Unit

from typing import List


class ChessGame:
    def __init__(self, board: chess.board.ChessBoard, moves: List[Move] = None, *args, **kwargs):
        self.board: chess.board.ChessBoard = board
        self.moves: List[Move] = moves if moves is not None else []

    def add_piece(self, square: Square, unit: chess.unit.Unit):
        self.board.set(square=square, unit=unit)

    def remove_piece(self, square: Square):
        self.board.set(square=square, unit=None)

    def _fit_path_max_steps_to_board(self, square: Square, path: Path):
        board_limited_max_steps: int = 0
        for _ in range(self.board.get_absolute_max_path_steps()):
            square: Square = square.offset(path.offset)
            if not self.board.is_valid_square(square=square):
                break
            board_limited_max_steps += 1
        return path.with_limited_max_steps(max_steps=board_limited_max_steps)

    def _fit_path_max_steps_to_blocked_path(self, square: Square, path: Path):
        max_steps: int = 0
        for _ in range(self.board.get_absolute_max_path_steps()):
            square: Square = square.offset(path.offset)
            if not self.board.is_valid_square(square=square):
                break
            if self.board.get(square=square) is not None:
                if path.allowed_path_types != AllowedMovementTypes.MOVE_ONLY:
                    max_steps += 1
                break
            max_steps += 1
        return path.with_limited_max_steps(max_steps=max_steps)





from typing import Set

from chess.color import Color
from chess.path import Path
from chess.offset import UP, DOWN, RIGHT, LEFT
from chess.piece import Knight
from chess.unit import Unit


UP_UP_RIGHT = Path(offset=UP * 2 + RIGHT, max_steps=1)
UP_RIGHT_RIGHT = Path(offset=UP + RIGHT * 2, max_steps=1)
DOWN_RIGHT_RIGHT = Path(offset=DOWN + RIGHT * 2, max_steps=1)
DOWN_DOWN_RIGHT = Path(offset=DOWN * 2 + RIGHT, max_steps=1)
DOWN_DOWN_LEFT = Path(offset=DOWN * 2 + LEFT, max_steps=1)
DOWN_LEFT_LEFT = Path(offset=DOWN + LEFT * 2, max_steps=1)
UP_LEFT_LEFT = Path(offset=UP + LEFT * 2, max_steps=1)
UP_UP_LEFT = Path(offset=UP * 2 + LEFT, max_steps=1)

KNIGHT_PATHS = {
    UP_UP_RIGHT,
    UP_RIGHT_RIGHT,
    DOWN_RIGHT_RIGHT,
    DOWN_DOWN_RIGHT,
    DOWN_DOWN_LEFT,
    DOWN_LEFT_LEFT,
    UP_LEFT_LEFT,
    UP_UP_LEFT,
}


class WhiteKnight(Unit, Knight):
    color: Color = Color.WHITE
    paths: Set[Path] = KNIGHT_PATHS


class BlackKnight(Unit, Knight):
    color: Color = Color.BLACK
    paths: Set[Path] = KNIGHT_PATHS





from dataclasses import dataclass


@dataclass
class ChessPiece:
    name: str = ''
    value: int = 0
    letter: str = ''
    symbol: str = ''

@dataclass
class King(ChessPiece):
    name: str = 'King'
    value: int = 0
    letter: str = 'K'
    symbol: str = '\u2654'

@dataclass
class Queen(ChessPiece):
    name: str = 'Queen'
    value: int = 9
    letter: str = 'Q'
    symbol: str = '\u2655'

@dataclass
class Rook(ChessPiece):
    name: str = 'Rook'
    value: int = 5
    letter: str = 'R'
    symbol: str = '\u2656'

@dataclass
class Bishop(ChessPiece):
    name: str = 'Bishop'
    value: int = 3
    letter: str = 'B'
    symbol: str = '\u2657'

@dataclass
class Knight(ChessPiece):
    name: str = 'Knight'
    value: int = 3
    letter: str = 'N'
    symbol: str = '\u2658'

@dataclass
class Pawn(ChessPiece):
    name: str = 'Pawn'
    value: int = 1
    letter: str = 'P'
    symbol: str = '\u2659'





from chess.color import Color
from chess.move import Move
import chess.notation.base as base


class AlgebraicNotation(base.Notation):
    @classmethod
    def get_indexes_from_square(cls, square: str) -> (int, int):
        if square[0] not in 'abcdefgh':
            raise ValueError('Invalid column')
        if square[1] not in '12345678':
            raise ValueError('Invalid row')
        return ord(square[0]) - ord('a'), int(square[1]) - 1




from .base import Unit
from .king import WhiteKing, BlackKing, KING_PATHS
from .queen import WhiteQueen, BlackQueen, QUEEN_PATHS
from .rook import WhiteRook, BlackRook, ROOK_PATHS
from .bishop import WhiteBishop, BlackBishop, BISHOP_PATHS
from .knight import WhiteKnight, BlackKnight, KNIGHT_PATHS
from .pawn import WhitePawn, BlackPawn





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['chess',
 'chess.board',
 'chess.game',
 'chess.gui',
 'chess.notation',
 'chess.unit']

package_data = \
{'': ['*']}

install_requires = \
['numpy>=1.23.2,<2.0.0']

setup_kwargs = {
    'name': '56kyle-pychess',
    'version': '0.1.1',
    'description': 'A python chess engine',
    'long_description': '\n# Pychess\n\n[![Status][status badge]][status badge]\n[![Tests][github actions badge]][github actions page]\n[![Codecov][codecov badge]][codecov page]\n[![Python Version][python version badge]][github page]\n[![License][license badge]][license]\n\n[code of conduct]: https://github.com/56kyle/pychess/blob/master/CODE_OF_CONDUCT.md\n[codecov badge]: https://codecov.io/gh/56kyle/pychess-instance/branch/master/graph/badge.svg\n[codecov page]: https://codecov.io/gh/56kyle/pychess-instance\n[contributor covenant badge]: https://img.shields.io/badge/Contributor%20Covenant-2.1-4baaaa.svg\n[github actions badge]: https://github.com/56kyle/pychess/workflows/Tests/badge.svg\n[github actions page]: https://github.com/56kyle/pychess/actions?workflow=Tests\n[github page]: https://github.com/56kyle/pychess\n[license badge]: https://img.shields.io/github/license/56kyle/pychess\n[license]: https://opensource.org/licenses/MIT\n[python version badge]: https://img.shields.io/pypi/pyversions/56kyle-pychess\n[status badge]: https://img.shields.io/pypi/status/56kyle-pychess\n\nA chess game made in Python as a proof of concept to validate the importance of testing to a friend\n\n\n[Pychess](#Pychess)\n- []()\n\n',
    'author': 'kyle',
    'author_email': '56kyleoliver@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/56kyle/pychess',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)




from typing import Set

from chess.color import Color
from chess.path import Path, AllowedMovementTypes
from chess.offset import RIGHT, LEFT, LINEAR, DIAGONAL
from chess.piece import King
from chess.unit import Unit


KING_SIDE_CASTLE_PATH = Path(offset=RIGHT * 2, max_steps=1, allowed_path_types=AllowedMovementTypes.MOVE_ONLY)
QUEEN_SIDE_CASTLE_PATH = Path(offset=LEFT * 3, max_steps=1, allowed_path_types=AllowedMovementTypes.MOVE_ONLY)
CASTLE_PATHS = {KING_SIDE_CASTLE_PATH, QUEEN_SIDE_CASTLE_PATH}

STANDARD_KING_PATHS = {Path(offset=offset, max_steps=1) for offset in LINEAR | DIAGONAL}
KING_PATHS = STANDARD_KING_PATHS | CASTLE_PATHS


class WhiteKing(Unit, King):
    color: Color = Color.WHITE
    paths: Set[Path] = KING_PATHS


class BlackKing(Unit, King):
    color: Color = Color.BLACK
    paths: Set[Path] = KING_PATHS




from typing import Set

from chess.color import Color
from chess.path import Path, INFINITE_STEPS
from chess.offset import DIAGONAL
from chess.piece import Bishop
from chess.unit import Unit


BISHOP_PATHS: Set[Path] = {Path(offset=offset, max_steps=INFINITE_STEPS) for offset in DIAGONAL}

class WhiteBishop(Unit, Bishop):
    color: Color = Color.WHITE
    paths: Set[Path] = BISHOP_PATHS


class BlackBishop(Unit, Bishop):
    color: Color = Color.BLACK
    paths: Set[Path] = BISHOP_PATHS





from typing import Set

from chess.color import Color
from chess.path import Path, AllowedMovementTypes
from chess.offset import UP, DOWN, UP_RIGHT, UP_LEFT, DOWN_RIGHT, DOWN_LEFT
from chess.piece import Pawn
from chess.unit import Unit


WHITE_PAWN_FIRST_PATH = Path(offset=UP * 2, max_steps=1, allowed_path_types=AllowedMovementTypes.MOVE_ONLY)
WHITE_PAWN_STANDARD_PATH = Path(offset=UP, max_steps=1, allowed_path_types=AllowedMovementTypes.MOVE_ONLY)
WHITE_PAWN_CAPTURE_PATHS = {
    Path(
        offset=offset,
        max_steps=1,
        allowed_path_types=AllowedMovementTypes.CAPTURE_ONLY,
    ) for offset in {UP_RIGHT, UP_LEFT}
}
WHITE_PAWN_PATHS = {WHITE_PAWN_FIRST_PATH, WHITE_PAWN_STANDARD_PATH} | WHITE_PAWN_CAPTURE_PATHS

BLACK_PAWN_FIRST_PATH = Path(offset=DOWN * 2, max_steps=1, allowed_path_types=AllowedMovementTypes.MOVE_ONLY)
BLACK_PAWN_STANDARD_PATH = Path(offset=DOWN, max_steps=1, allowed_path_types=AllowedMovementTypes.MOVE_ONLY)
BLACK_PAWN_CAPTURE_PATHS = {
    Path(
        offset=offset,
        max_steps=1,
        allowed_path_types=AllowedMovementTypes.CAPTURE_ONLY
    ) for offset in {DOWN_RIGHT, DOWN_LEFT}
}
BLACK_PAWN_PATHS = {BLACK_PAWN_FIRST_PATH, BLACK_PAWN_STANDARD_PATH} | BLACK_PAWN_CAPTURE_PATHS


class WhitePawn(Unit, Pawn):
    color: Color = Color.WHITE
    paths: Set[Path] = WHITE_PAWN_PATHS


class BlackPawn(Unit, Pawn):
    color: Color = Color.BLACK
    paths: Set[Path] = BLACK_PAWN_PATHS





from dataclasses import dataclass
from enum import Enum

from chess.offset import Offset


INFINITE_STEPS = 0


class AllowedMovementTypes(Enum):
    MOVE_ONLY = 1
    CAPTURE_ONLY = 2
    BOTH = 3


@dataclass(frozen=True)
class Path:
    offset: Offset = Offset(dy=0, dx=0)
    max_steps: int = INFINITE_STEPS
    allowed_path_types: AllowedMovementTypes = AllowedMovementTypes.BOTH

    def with_limited_max_steps(self, max_steps: int) -> 'Path':
        new_max_steps: int = min(self.max_steps, max_steps) if self.max_steps != INFINITE_STEPS else max_steps
        return Path(self.offset, new_max_steps, self.allowed_path_types)

    def limit_to_shape(self, height: int, width: int) -> 'Path':
        return Path(self.offset, self._get_shape_limited_steps(height, width), self.allowed_path_types)

    def _get_shape_limited_steps(self, height: int, width: int) -> int:
        height_limited_steps: int = self._get_height_limited_steps(height)
        width_limited_steps: int = self._get_width_limited_steps(width)
        return min(height_limited_steps, width_limited_steps)

    def limit_to_height(self, height: int) -> 'Path':
        return Path(self.offset, self._get_height_limited_steps(height), self.allowed_path_types)

    def _get_height_limited_steps(self, height: int) -> int:
        height_limited_steps: int = height // abs(self.offset.dy)
        if self.max_steps == INFINITE_STEPS:
            return height_limited_steps
        return min(self.max_steps, height_limited_steps)

    def limit_to_width(self, width: int) -> 'Path':
        return Path(self.offset, self._get_width_limited_steps(width), self.allowed_path_types)

    def _get_width_limited_steps(self, width: int) -> int:
        width_limited_steps: int = width // abs(self.offset.dx)
        if self.max_steps == INFINITE_STEPS:
            return width_limited_steps
        return min(self.max_steps, width_limited_steps)


